package
{
	import net.flashpunk.*;
	import net.flashpunk.graphics.*;
	import net.flashpunk.masks.*;
	import net.flashpunk.utils.*;
	
	import flash.display.*;
	import flash.geom.*;
	
	public class Level extends World
	{
		public var timer: int = 0;
		
		public var red: Graphic;
		public var green: Graphic;
		
		public static var colour: uint = 0xFFFF0000;
		
		public var inverseBuffer: BitmapData;
		
		public function Level()
		{
			var bitmap: BitmapData = new BitmapData(FP.width, FP.height, false, 0xFF000000);
			
			FP.rect.width = FP.width;
			FP.rect.height = 5;
			FP.rect.x = 0;
			
			for (var i: int = 0; i < 48; i++) {
				FP.rect.y = i*10;
				bitmap.fillRect(FP.rect, 0xFFFF0000);
			}
			
			red = new Backdrop(bitmap, false, true);
			red.scrollX = 0;
			red.visible = true;
			
			bitmap = new BitmapData(FP.width, FP.height, false, 0xFF000000);
			
			FP.rect.width = 5;
			FP.rect.height = FP.height;
			FP.rect.y = 0;
			
			for (i = 0; i < 48; i++) {
				FP.rect.x = i*10;
				bitmap.fillRect(FP.rect, 0xFF00FF00);
			}
			
			green = new Backdrop(bitmap, true, false);
			green.scrollY = 0;
			
			green.visible = false;
			
			colour = 0xFFFF0000;
			
			inverseBuffer = new BitmapData(FP.width, FP.height, true, 0);
			
			FP.rect.x = 160;
			FP.rect.y = 160;
			FP.rect.width = 160;
			FP.rect.height = 160;
			
			inverseBuffer.fillRect(FP.rect, 0xFFFFFFFF);
			
			add(new Player());
			
			add(new Square());
			add(new Square());
		}
		
		public override function update (): void
		{
			timer++;
			
			if (timer % 600 == 0) {
				red.visible = !red.visible;
				green.visible = !green.visible;
				colour ^= 0xFFFF00;
			}
			
			super.update();
			
			//FP.camera.x += (Input.mouseX - 240) * 0.01;
			//FP.camera.y += (Input.mouseY - 240) * 0.01;
		}
		
		public override function render (): void
		{
			var tmp: BitmapData = FP.buffer;
			
			FP.buffer = inverseBuffer;
			
			inverseBuffer.fillRect(FP.bounds, 0);
			
			Draw.setTarget(inverseBuffer, FP.camera);
			
			super.render();
			
			FP.buffer = tmp;
			
			p1.x = p1.y = 0;
			p2.x = FP.camera.x;
			p2.y = FP.camera.y;
			
			if (red.visible) { red.render(p1, p2); }
			if (green.visible) { green.render(p1, p2); }
			
			inverseBuffer.lock();
			
			draw(inverseBuffer, 0, 0);
			
			//FP.buffer.copyPixels(inverseBuffer, FP.bounds, FP.zero, null, null, true);
			
			inverseBuffer.unlock();
		}
		
		public var p1: Point = new Point();
		public var p2: Point = new Point();
		
		public static function draw (image: BitmapData, x: Number, y: Number, clipRect: Rectangle = null): void
		{
			if (! clipRect) {
				clipRect = image.rect;
			}
			
			for (var i: int = 0; i < clipRect.width; i++) {
				for (var j: int = 0; j < clipRect.height; j++) {
					var x2: int = clipRect.x + i;
					var y2: int = clipRect.y + j;
					
					var c: uint = image.getPixel32(x2, y2);
					
					if (c < uint(0x80000000)) { continue; }
					
					x2 += x;
					y2 += y;
					
					c = FP.buffer.getPixel32(x2, y2) & 0xFFFFFF;
					
					if (c == 0) {
						c = colour;
					} else {
						c = 0;
					}
					
					FP.buffer.setPixel(x2, y2, c);
				}
			}
		}
	}
}
